package web.controller;

import db.annotation.TableInfo;
import db.config.HibernateConfig;
import db.form.DBAction;
import db.form.JsonResult;
import db.form.Type;
import db.model.AbstractModel;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.criterion.DetachedCriteria;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ResolvableType;
import org.springframework.dao.DataAccessException;
import org.springframework.http.MediaType;
import org.springframework.util.ClassUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import web.service.BaseService;

/**
 * Created by reborn on 2017/9/14.
 */
public abstract class BaseController<T extends AbstractModel,A extends T,E extends BaseService>{
    protected static Logger log=LogManager.getLogger();

    @Autowired
    protected E service;

    private ResolvableType resolvableType=ResolvableType.forClass(ClassUtils.getUserClass(this.getClass()));

    protected final String tableName;


    public BaseController() {
        if(getResolvableType()!=AbstractModel.class){
            TableInfo tableInfo=HibernateConfig.getTableNote(getResolvableType());
            if(tableInfo!=null&&StringUtils.isNotEmpty(tableInfo.comment())){
                tableName=tableInfo.comment();
            }else {
                tableName=getResolvableType().getName();
            }
        }else{
            tableName=null;
        }
    }

    @ResponseBody
    @RequestMapping(value = "{action}", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public JsonResult curd(@PathVariable DBAction action, T model) {
        log.info(tableName+ "进行" + action.getCh() + "操作请求");
        if(checkAction(action)) {
            try {
                return new JsonResult<>(service.curd(action, model), Type.success, action);
            }catch (DataAccessException e){
                log.error(e);
                return new JsonResult<>("非法操作", Type.fail, action);
            }
        }else{
            return new JsonResult<>(tableName + "不允许" + action.getCh() + "操作", Type.fail,action);
        }

    }

    @ResponseBody
    @RequestMapping(value = "batch/{action}", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public  JsonResult find(@PathVariable DBAction action, A queryCommand, @RequestParam Integer firstResult, @RequestParam Integer maxResults){
        log.info(tableName + "进行批量" + action.getCh() + "操作请求");
        if(checkAction(action)) {
            try {

                return new JsonResult<>(service.find(getDetachedCriteria(queryCommand), firstResult, maxResults), Type.success, action);
            } catch (DataAccessException e) {
                log.error(e);
                return new JsonResult<>("非法操作", Type.fail, action);
            }
        }else{
            return new JsonResult<>(tableName + "不允许" + action.getCh() + "操作", Type.fail,action);
        }

    }

    public Class<?> getResolvableType() {
        return getResolvableType(0);
    }

    public Class<?> getResolvableType(int index) {
        return resolvableType.getSuperType().getGeneric(index).resolve();
    }

    protected DetachedCriteria getDetachedCriteria(A queryCommand){
        return DetachedCriteria.forClass(getResolvableType());
    }

    protected abstract boolean checkAction(DBAction action);
}
