package web.service;

import db.annotation.Model;
import db.config.HibernateConfig;
import db.form.DBAction;
import db.model.AbstractModel;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Projections;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate5.HibernateTemplate;
import org.springframework.util.ClassUtils;

import java.util.List;

/**
 * Created by reborn on 2017/7/28.
 */

public abstract class BaseService{

    protected static Logger log=LogManager.getLogger();

    protected HibernateTemplate hibernateTemplate;

    public final HibernateTemplate getHibernateTemplate() {
        if(hibernateTemplate==null) {
            hibernateTemplate=HibernateConfig.get(ClassUtils.getUserClass(this.getClass()).getAnnotation(Model.class).value());
        }
        return hibernateTemplate;
    }

    public final <T extends AbstractModel> T curd(DBAction action, T command) throws DataAccessException {
        switch (action) {
            case C:
                getHibernateTemplate().save(command);break;
            case U:
                getHibernateTemplate().update(command);break;
            case R:
                T model=(T)(getHibernateTemplate().get(command.getClass(), command.primaryKey()));
                if(model!=null){
                    return model.cast();
                }
            case D:
                getHibernateTemplate().delete(command);break;
            default:
                throw new RuntimeException("非法操作：" + action);
        }
        return null;

    }

    public final List find(DetachedCriteria criteria, int firstResult,int maxResults){
        return getHibernateTemplate().findByCriteria(criteria,firstResult,maxResults<getMaxResults()?maxResults:getMaxResults());
    }

    public final List find(DetachedCriteria criteria){
        return getHibernateTemplate().findByCriteria(criteria);
    }

    /**
     * 限制分页查询最大显示数
     * @return
     */
    protected int getMaxResults(){
        return 100;
    }

    /**
     * 根据查询条件统计记录数
     * @param criteria
     * @return
     */
    public Long rowCount(DetachedCriteria criteria){
        return getHibernateTemplate().executeWithNativeSession(session -> (Long) criteria.setProjection(Projections.rowCount()).getExecutableCriteria(session).uniqueResult());
    }


}
